@@ -28,7 +28,6 @@
 #include <QGraphicsScene>
 #include <QScrollBar>
 #include <QAction>
-#include <QDebug>
 #include <QIcon>
 
 #include <KLocalizedString>
@@ -82,19 +81,24 @@ ImageViewer::~ImageViewer()
 // ------------------------------------------------------------------------
 void ImageViewer::setQImage(QImage *img)
 {
-    if (img == 0) {
+    if (img == nullptr) {
         return;
     }
 
     d->img = img;
-    d->scene->setSceneRect(0, 0, img->width(), img->height());
+    const auto dpr = devicePixelRatioF();
+    d->img->setDevicePixelRatio(dpr);
+    d->scene->setSceneRect(0, 0, img->width() / dpr, img->height() / dpr);
 }
 
 // ------------------------------------------------------------------------
 void ImageViewer::drawBackground(QPainter *painter, const QRectF &rect)
 {
     painter->fillRect(rect, QColor(0x70, 0x70, 0x70));
-    painter->drawImage(rect, *d->img, rect);
+    QRectF r = rect & sceneRect();
+    const auto dpr = d->img->devicePixelRatio();
+    QRectF srcRect = QRectF(r.topLeft() * dpr, r.size() * dpr);
+    painter->drawImage(r, *d->img, srcRect);
 }
 
 // ------------------------------------------------------------------------
@@ -112,20 +116,23 @@ void ImageViewer::zoomOut()
 // ------------------------------------------------------------------------
 void ImageViewer::zoomActualSize()
 {
-    setMatrix(QMatrix());
+    resetTransform();
 }
 
 // ------------------------------------------------------------------------
 void ImageViewer::zoom2Fit()
 {
-    fitInView(d->img->rect(), Qt::KeepAspectRatio);
+    QRectF r = d->img->rect();
+    const auto dpr = d->img->devicePixelRatio();
+    r = QRectF(r.topLeft() / dpr, r.size() / dpr);
+    fitInView(r, Qt::KeepAspectRatio);
 }
 
 // ------------------------------------------------------------------------
 void ImageViewer::wheelEvent(QWheelEvent *e)
 {
     if (e->modifiers() == Qt::ControlModifier) {
-        if (e->delta() > 0) {
+        if (e->angleDelta().y() > 0) {
             zoomIn();
         }
         else {
