@@ -26,20 +26,42 @@
 #ifndef SAVE_LOCATION_H
 #define SAVE_LOCATION_H
 
-#include "ui_SaveLocation.h"
-
 #include <QDialog>
 
-class SaveLocation : public QDialog, public Ui_SaveLocation
+class Ui_SaveLocation;
+class QShowEvent;
+
+class SaveLocation : public QDialog
 {
     Q_OBJECT
 public:
-    SaveLocation(QWidget *parent = 0);
+    explicit SaveLocation(QWidget *parent = nullptr);
     ~SaveLocation();
 
+    QUrl folderUrl() const;
+    QString imagePrefix() const;
+    QString imageFormat() const;
+    int startNumber() const;
+    int startNumberMax() const;
+
+    void setFolderUrl(const QUrl &url);
+    void setImagePrefix(const QString &prefix);
+    void setImageFormat(const QString &format);
+    void setStartNumber(int number);
+
+    void setImageFormats(const QStringList &formats);
+
+    void setOpenRequesterOnShow(bool open);
+
+protected:
+    void showEvent(QShowEvent *event) override;
+
 private Q_SLOTS:
     void updateGui();
-    void getDir();
+
+private:
+    Ui_SaveLocation *m_ui = nullptr;
+    bool m_openRequesterOnShow = false;
 };
 
 #endif
