@@ -24,8 +24,9 @@
 
 #include "skanlite.h"
 
-#include "KSaneImageSaver.h"
 #include "SaveLocation.h"
+#include "showimagedialog.h"
+#include "SkanliteImageSaver.h"
 
 #include <QApplication>
 #include <QScrollArea>
@@ -36,10 +37,11 @@
 #include <QComboBox>
 #include <QMessageBox>
 #include <QTemporaryFile>
-#include <QDebug>
 #include <QImageWriter>
 #include <QMimeType>
 #include <QMimeDatabase>
+#include <QCloseEvent>
+#include <QProgressBar>
 
 #include <KAboutApplicationDialog>
 #include <KLocalizedString>
@@ -52,6 +54,8 @@
 #include <KConfigGroup>
 #include <KHelpClient>
 
+#include <skanlite_debug.h>
+
 #include <errno.h>
 
 Skanlite::Skanlite(const QString &device, QWidget *parent)
@@ -76,8 +80,23 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
     connect(m_ksanew, &KSaneWidget::availableDevices, this, &Skanlite::availableDevices);
     connect(m_ksanew, &KSaneWidget::userMessage, this, &Skanlite::alertUser);
     connect(m_ksanew, &KSaneWidget::buttonPressed, this, &Skanlite::buttonPressed);
+    connect(m_ksanew, &KSaneWidget::scanDone, this, [this](){
+        if (!m_pendingApplyScanOpts.isEmpty()) {
+            applyScannerOptions(m_pendingApplyScanOpts);
+        }
+    });
+
+    m_saveProgressBar = new QProgressBar(this);
+    m_saveProgressBar->setVisible(false);
+    m_saveProgressBar->setFormat(i18n("Saving: %v kB"));
+    m_saveProgressBar->setTextVisible(true);
+
+    m_saveUpdateTimer.setInterval(200);
+    m_saveUpdateTimer.setSingleShot(false);
+    connect(&m_saveUpdateTimer, &QTimer::timeout, this, &Skanlite::updateSaveProgress);
 
     mainLayout->addWidget(m_ksanew);
+    mainLayout->addWidget(m_saveProgressBar);
     mainLayout->addWidget(dlgButtonBoxBottom);
 
     m_ksanew->initGetDeviceList();
@@ -117,7 +136,7 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
             m_filterList.append(QString::fromLatin1(ba));
         }
 
-        qDebug() << m_filterList;
+        qCDebug(SKANLITE_LOG) << m_filterList;
 
         // Put first class citizens at first place
         m_filterList.removeAll(QLatin1String("image/jpeg"));
@@ -143,7 +162,7 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
         }
 
         m_settingsUi.imgFormat->addItems(m_typeList);
-        m_saveLocation->u_imgFormat->addItems(m_typeList);
+        m_saveLocation->setImageFormats(m_typeList);
 
         mainLayout->addWidget(settingsWidget);
 
@@ -156,31 +175,30 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
 
         m_settingsDialog->setWindowTitle(i18n("Skanlite Settings"));
 
-        connect(m_settingsUi.getDirButton, &QPushButton::clicked, this, &Skanlite::getDir);
         connect(m_settingsUi.revertOptions, &QPushButton::clicked, this, &Skanlite::defaultScannerOptions);
         readSettings();
 
         // default directory for the save dialog
-        m_saveLocation->u_urlRequester->setUrl(QUrl::fromUserInput(m_settingsUi.saveDirLEdit->text()));
-        m_saveLocation->u_imgPrefix->setText(m_settingsUi.imgPrefix->text());
-        m_saveLocation->u_imgFormat->setCurrentText(m_settingsUi.imgFormat->currentText());
+        m_saveLocation->setFolderUrl(m_settingsUi.saveDirRequester->url());
+        m_saveLocation->setImagePrefix(m_settingsUi.imgPrefix->text());
+        m_saveLocation->setImageFormat(m_settingsUi.imgFormat->currentText());
     }
 
     // open the scan device
     if (m_ksanew->openDevice(device) == false) {
-        QString dev = m_ksanew->selectDevice(0);
+        QString dev = m_ksanew->selectDevice(nullptr);
         if (dev.isEmpty()) {
             // either no scanner was found or then cancel was pressed.
             exit(0);
         }
         if (m_ksanew->openDevice(dev) == false) {
             // could not open a scanner
-            KMessageBox::sorry(0, i18n("Opening the selected scanner failed."));
+            KMessageBox::sorry(nullptr, i18n("Opening the selected scanner failed."));
             exit(1);
         }
         else {
             setWindowTitle(i18nc("@title:window %1 = scanner maker, %2 = scanner model", "%1 %2 - Skanlite", m_ksanew->make(), m_ksanew->model()));
-            m_deviceName = QString::fromLatin1("%1:%2").arg(m_ksanew->make()).arg(m_ksanew->model());
+            m_deviceName = QString::fromLatin1("%1:%2").arg(m_ksanew->make(), m_ksanew->model());
         }
     }
     else {
@@ -189,27 +207,9 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
     }
 
     // prepare the Show Image Dialog
-    {
-        m_showImgDialog = new QDialog(this);
-
-        QVBoxLayout *mainLayout = new QVBoxLayout(m_showImgDialog);
-
-        QDialogButtonBox *imgButtonBox = new QDialogButtonBox(m_showImgDialog);
-        // "Close" (now Discard) and "User1"=Save
-        imgButtonBox->setStandardButtons(QDialogButtonBox::Discard | QDialogButtonBox::Save);
-
-        mainLayout->addWidget(&m_imageViewer);
-        mainLayout->addWidget(imgButtonBox);
-
-        m_showImgDialogSaveButton = imgButtonBox->button(QDialogButtonBox::Save);
-        m_showImgDialogSaveButton->setDefault(true); // still needed?
-
-        m_showImgDialog->resize(640, 480);
-        connect(imgButtonBox, &QDialogButtonBox::accepted, this, &Skanlite::saveImage);
-        //connect(imgButtonBox, &QDialogButtonBox::accepted, m_showImgDialog, &QDialog::accept);
-        connect(imgButtonBox->button(QDialogButtonBox::Discard), &QPushButton::clicked, m_ksanew, &KSaneWidget::scanCancel);
-        connect(imgButtonBox->button(QDialogButtonBox::Discard), &QPushButton::clicked, m_showImgDialog, &QDialog::reject);
-    }
+    m_showImgDialog = new ShowImageDialog(this);
+    connect(m_showImgDialog, &ShowImageDialog::saveRequested, this, &Skanlite::saveImage);
+    connect(m_showImgDialog, &ShowImageDialog::rejected, m_ksanew, &KSaneWidget::scanCancel);
 
     // save the default sane options for later use
     m_ksanew->getOptVals(m_defaultScanOpts);
@@ -224,6 +224,7 @@ Skanlite::Skanlite(const QString &device, QWidget *parent)
     if (m_dbusInterface.setupDBusInterface()) {
         // D-Bus related slots
         connect(&m_dbusInterface, &DBusInterface::requestedScan, m_ksanew, &KSaneWidget::scanFinal);
+        connect(&m_dbusInterface, &DBusInterface::requestedPreview, m_ksanew, &KSaneWidget::startPreviewScan);
         connect(&m_dbusInterface, &DBusInterface::requestedScanCancel, m_ksanew, &KSaneWidget::scanCancel);
         connect(&m_dbusInterface, &DBusInterface::requestedSetScannerOptions, this, &Skanlite::setScannerOptions);
         connect(&m_dbusInterface, &DBusInterface::requestedSetSelection, this, &Skanlite::setSelection);
@@ -276,10 +277,10 @@ void Skanlite::saveWindowSize()
 static void perrorMessageBox(const QString &text)
 {
     if (errno != 0) {
-        KMessageBox::sorry(0, i18n("%1: %2", text, QString::fromLocal8Bit(strerror(errno))));
+        KMessageBox::sorry(nullptr, i18n("%1: %2", text, QString::fromLocal8Bit(strerror(errno))));
     }
     else {
-        KMessageBox::sorry(0, text);
+        KMessageBox::sorry(nullptr, text);
     }
 }
 
@@ -295,7 +296,7 @@ void Skanlite::readSettings(void)
     if (m_settingsUi.saveModeCB->currentIndex() != SaveModeAskFirst) {
         m_firstImage = false;
     }
-    m_settingsUi.saveDirLEdit->setText(saving.readEntry("Location", QDir::homePath()));
+    m_settingsUi.saveDirRequester->setUrl(saving.readEntry("Location", QUrl(QDir::homePath())));
     m_settingsUi.imgPrefix->setText(saving.readEntry("NamePrefix", i18nc("prefix for auto naming", "Image-")));
     m_settingsUi.imgFormat->setCurrentText(saving.readEntry("ImgFormat", "png"));
     m_settingsUi.imgQuality->setValue(saving.readEntry("ImgQuality", 90));
@@ -327,7 +328,7 @@ void Skanlite::showSettingsDialog(void)
         // save the settings
         KConfigGroup saving(KSharedConfig::openConfig(), "Image Saving");
         saving.writeEntry("SaveMode", m_settingsUi.saveModeCB->currentIndex());
-        saving.writeEntry("Location", m_settingsUi.saveDirLEdit->text());
+        saving.writeEntry("Location", m_settingsUi.saveDirRequester->url());
         saving.writeEntry("NamePrefix", m_settingsUi.imgPrefix->text());
         saving.writeEntry("ImgFormat", m_settingsUi.imgFormat->currentText());
         saving.writeEntry("SetQuality", m_settingsUi.setQuality->isChecked());
@@ -352,9 +353,9 @@ void Skanlite::showSettingsDialog(void)
         m_ksanew->enableAutoSelect(!m_settingsUi.u_disableSelections->isChecked());
 
         // pressing OK in the settings dialog means use those settings.
-        m_saveLocation->u_urlRequester->setUrl(QUrl::fromUserInput(m_settingsUi.saveDirLEdit->text()));
-        m_saveLocation->u_imgPrefix->setText(m_settingsUi.imgPrefix->text());
-        m_saveLocation->u_imgFormat->setCurrentText(m_settingsUi.imgFormat->currentText());
+        m_saveLocation->setFolderUrl(m_settingsUi.saveDirRequester->url());
+        m_saveLocation->setImagePrefix(m_settingsUi.imgPrefix->text());
+        m_saveLocation->setImageFormat(m_settingsUi.imgFormat->currentText());
 
         m_firstImage = true;
     }
@@ -376,9 +377,8 @@ void Skanlite::imageReady(QByteArray &data, int w, int h, int bpl, int f)
     if (m_settingsUi.showB4Save->isChecked() == true) {
         /* copy the image data into m_img and show it*/
         m_img = m_ksanew->toQImageSilent(data, w, h, bpl, (KSaneIface::KSaneWidget::ImageFormat)f);
-        m_imageViewer.setQImage(&m_img);
-        m_imageViewer.zoom2Fit();
-        m_showImgDialogSaveButton->setFocus();
+        m_showImgDialog->setQImage(&m_img);
+        m_showImgDialog->zoom2Fit();
         m_showImgDialog->exec();
         // save has been done as a result of save or then we got cancel
     }
@@ -388,55 +388,78 @@ void Skanlite::imageReady(QByteArray &data, int w, int h, int bpl, int f)
     }
 }
 
+bool urlExists(const QUrl& url)
+{
+    if (url.isLocalFile()) {
+        if (!QFileInfo::exists(url.toLocalFile())) {
+            return false;
+        }
+    }
+    else {
+        KIO::StatJob *statJob = KIO::statDetails(url, KIO::StatJob::DestinationSide);
+        KJobWidgets::setWindow(statJob, QApplication::activeWindow());
+        if (!statJob->exec()) {
+            return false;
+        }
+    }
+    return true;
+}
+
 void Skanlite::saveImage()
 {
-    // ask the first time if we are in "ask on first" mode
-    if ((m_settingsUi.saveModeCB->currentIndex() == SaveModeAskFirst) && m_firstImage) {
-        if (m_saveLocation->exec() != QFileDialog::Accepted) {
-            m_ksanew->scanCancel(); // In case we are cancelling a document feeder scan
-            return;
+    QUrl dirUrl = m_saveLocation->folderUrl();
+    bool dirExists = urlExists(dirUrl);
+
+    // Ask the first time if we are in "ask on first" mode
+    if (m_settingsUi.saveModeCB->currentIndex() == SaveModeAskFirst) {
+        while (m_firstImage || !dirExists) {
+            m_saveLocation->setOpenRequesterOnShow(!dirExists);
+            if (m_saveLocation->exec() != QFileDialog::Accepted) {
+                m_ksanew->scanCancel(); // In case we are cancelling a document feeder scan
+                return;
+            }
+            dirUrl = m_saveLocation->folderUrl();
+            dirExists = urlExists(dirUrl); // check that we actually got an existing folder
+            m_firstImage = false;
         }
-        m_firstImage = false;
+    }
+    else if (!dirExists) {
+        // The save-folder from settings does not exist! Use the users home directory.
+        dirUrl = QUrl::fromUserInput(QDir::homePath() + QLatin1Char('/'));
+        m_saveLocation->setFolderUrl(dirUrl);
     }
 
-    QString dir = QDir::cleanPath(m_saveLocation->u_urlRequester->url().url()).append(QLatin1Char('/')); //make sure whole value is processed as path to directory
-    QString prefix = m_saveLocation->u_imgPrefix->text();
-    QString imgFormat = m_saveLocation->u_imgFormat->currentText().toLower();
-    int fileNumber = m_saveLocation->u_numStartFrom->value();
+
+    QString prefix = m_saveLocation->imagePrefix();
+    QString imgFormat = m_saveLocation->imageFormat().toLower();
+    int fileNumber = m_saveLocation->startNumber();
     QStringList filterList = m_filterList;
+    bool enforceSavingAsPng16bit = false;
     if ((m_format == KSaneIface::KSaneWidget::FormatRGB_16_C) ||
-            (m_format == KSaneIface::KSaneWidget::FormatGrayScale16)) {
+        (m_format == KSaneIface::KSaneWidget::FormatGrayScale16))
+    {
         filterList = m_filter16BitList;
+        enforceSavingAsPng16bit = true;
         if (imgFormat != QLatin1String("png")) {
             imgFormat = QLatin1String("png");
             KMessageBox::information(this, i18n("The image will be saved in the PNG format, as Skanlite only supports saving 16 bit color images in the PNG format."));
         }
     }
 
-    //qDebug() << dir << prefix << imgFormat;
-
     // find next available file name for name suggestion
     QUrl fileUrl;
     QString fname;
-    for (int i = fileNumber; i <= m_saveLocation->u_numStartFrom->maximum(); ++i) {
+    for (int i = fileNumber; i <= m_saveLocation->startNumberMax(); ++i) {
         fname = QString::fromLatin1("%1%2.%3")
                 .arg(prefix)
                 .arg(i, 4, 10, QLatin1Char('0'))
                 .arg(imgFormat);
 
-        fileUrl = QUrl::fromUserInput(QStringLiteral("%1%2").arg(dir, fname));
-        //qDebug() << fileUrl;
-        if (fileUrl.isLocalFile()) {
-            if (!QFileInfo(fileUrl.toLocalFile()).exists()) {
-                break;
-            }
-        }
-        else {
-            KIO::StatJob *statJob = KIO::stat(fileUrl, KIO::StatJob::DestinationSide, 0);
-            KJobWidgets::setWindow(statJob, QApplication::activeWindow());
-            if (!statJob->exec()) {
-                break;
-            }
+        fileUrl = dirUrl;
+        fileUrl.setPath(fileUrl.path() + fname);
+        fileUrl = fileUrl.adjusted(QUrl::NormalizePathSegments);
+        if (!urlExists(fileUrl)) {
+            break;
         }
     }
 
@@ -456,39 +479,12 @@ void Skanlite::saveImage()
         QString currentMimeFilter = QLatin1String("image/") + imgFormat;
         saveDialog.setMimeTypeFilters(actualFilterList);
         saveDialog.selectMimeTypeFilter(currentMimeFilter);
-        //qDebug() << fileUrl.url() << fileUrl.toLocalFile() << currentMimeFilter;
 
-        do {
-            if (saveDialog.exec() != QFileDialog::Accepted) {
-                return;
-            }
+        if (saveDialog.exec() != QFileDialog::Accepted) {
+            return;
+        }
 
-            fileUrl = saveDialog.selectedUrls()[0];
-
-            bool exists;
-            if (fileUrl.isLocalFile()) {
-                exists = QFileInfo(fileUrl.toLocalFile()).exists();
-            }
-            else {
-                KIO::StatJob *statJob = KIO::stat(fileUrl, KIO::StatJob::DestinationSide, 0);
-                KJobWidgets::setWindow(statJob, QApplication::activeWindow());
-                exists = statJob->exec();
-            }
-            if (exists) {
-                if (KMessageBox::warningContinueCancel(this,
-                    i18n("Do you want to overwrite \"%1\"?", fileUrl.fileName()),
-                    QString(),
-                    KStandardGuiItem::overwrite(),
-                    KStandardGuiItem::cancel(),
-                    QLatin1String("editorWindowSaveOverwrite")
-                ) ==  KMessageBox::Continue) {
-                    break;
-                }
-            }
-            else {
-                break;
-            }
-        } while (true);
+        fileUrl = saveDialog.selectedUrls().at(0);
     }
 
     m_firstImage = false;
@@ -499,13 +495,11 @@ void Skanlite::saveImage()
         quality = m_settingsUi.imgQuality->value();
     }
 
-    //qDebug() << "suffix" << QFileInfo(fileUrl.fileName()).suffix();
     QString localName;
-
     QString suffix = QFileInfo(fileUrl.fileName()).suffix();
-    const char *fileFormat = nullptr;
+    QString fileFormat;
     if (suffix.isEmpty()) {
-        fileFormat = "png";
+        fileFormat = QLatin1String("png");
     }
 
     if (!fileUrl.isLocalFile()) {
@@ -523,30 +517,60 @@ void Skanlite::saveImage()
         localName = fileUrl.toLocalFile();
     }
 
+    SkanliteImageSaver *imageSaver = new SkanliteImageSaver(this);
+    connect(imageSaver, &SkanliteImageSaver::imageSaved, this, &Skanlite::imageSaved);
+
     // Save
-    if ((m_format == KSaneIface::KSaneWidget::FormatRGB_16_C) ||
-        (m_format == KSaneIface::KSaneWidget::FormatGrayScale16)) {
-        KSaneImageSaver saver;
-        if (saver.savePngSync(localName, m_data, m_width, m_height, m_format, m_ksanew->currentDPI())) {
-            m_showImgDialog->close(); // closing the window if it is closed should not be a problem.
-        }
-        else {
-            perrorMessageBox(i18n("Failed to save image"));
-            return;
-        }
+    if (enforceSavingAsPng16bit) {
+        imageSaver->save16BitPng(fileUrl, localName, m_data, m_width, m_height, m_bytesPerLine, (int)m_ksanew->currentDPI(), m_format, fileFormat, quality);
+    } else {
+        imageSaver->saveQImage(fileUrl, localName, m_data, m_width, m_height, m_bytesPerLine, (int)m_ksanew->currentDPI(), m_format, fileFormat, quality);
     }
-    else  {
-        // create the image if needed.
-        if (m_img.width() < 1) {
-            m_img = m_ksanew->toQImage(m_data, m_width, m_height, m_bytesPerLine, (KSaneIface::KSaneWidget::ImageFormat)m_format);
-        }
-        if (m_img.save(localName, fileFormat, quality)) {
-            m_showImgDialog->close(); // calling close() on a closed window does nothing.
-        }
-        else {
-            perrorMessageBox(i18n("Failed to save image"));
-            return;
-        }
+
+    m_showImgDialog->close(); // calling close() on a closed window does nothing.
+
+    // Disable parts of the interface and indicate that we are saving the image
+    m_currentSaveUrl = fileUrl;
+    m_ksanew->setDisabled(true);
+    m_saveProgressBar->setMaximum(0);
+    m_saveProgressBar->setValue(0);
+    m_saveProgressBar->setVisible(true);
+    m_saveUpdateTimer.start();
+
+    // Save the file base name without number
+    QString baseName = QFileInfo(fileUrl.fileName()).completeBaseName();
+    while ((!baseName.isEmpty()) && (baseName[baseName.size() - 1].isNumber())) {
+        baseName.remove(baseName.size() - 1, 1);
+    }
+    m_saveLocation->setImagePrefix(baseName);
+
+    // Save the number
+    if (fileNumber) {
+        m_saveLocation->setStartNumber(fileNumber + 1);
+    }
+
+    if (m_settingsUi.saveModeCB->currentIndex() == SaveModeManual) {
+        // Save last used dir, prefix and suffix.
+        m_saveLocation->setFolderUrl(KIO::upUrl(fileUrl));
+        m_saveLocation->setImageFormat(QFileInfo(fileUrl.fileName()).suffix());
+    }
+
+
+}
+
+void Skanlite::updateSaveProgress()
+{
+    QFileInfo saveInfo(m_currentSaveUrl.toLocalFile());
+    quint64 size = saveInfo.size()/1024;
+    m_saveProgressBar->setMaximum(size);
+    m_saveProgressBar->setValue(size);
+}
+
+void Skanlite::imageSaved(const QUrl &fileUrl, const QString &localName, bool success)
+{
+    if (!success) {
+        perrorMessageBox(i18n("Failed to save image"));
+        return;
     }
 
     if (!fileUrl.isLocalFile()) {
@@ -558,7 +582,7 @@ void Skanlite::saveImage()
         tmpFile.close();
         tmpFile.remove();
         if (!ok) {
-            KMessageBox::sorry(0, i18n("Failed to upload image"));
+            KMessageBox::sorry(nullptr, i18n("Failed to upload image"));
         }
         else {
             emit m_dbusInterface.imageSaved(fileUrl.toString());
@@ -567,34 +591,13 @@ void Skanlite::saveImage()
     else {
         emit m_dbusInterface.imageSaved(localName);
     }
+    m_ksanew->setDisabled(false);
+    m_saveUpdateTimer.stop();
+    m_saveProgressBar->setVisible(false);
 
-    // Save the file base name without number
-    QString baseName = QFileInfo(fileUrl.fileName()).completeBaseName();
-    while ((!baseName.isEmpty()) && (baseName[baseName.size() - 1].isNumber())) {
-        baseName.remove(baseName.size() - 1, 1);
-    }
-    m_saveLocation->u_imgPrefix->setText(baseName);
-
-    // Save the number
-    QString fileNumStr = QFileInfo(fileUrl.fileName()).completeBaseName();
-    fileNumStr.remove(baseName);
-    fileNumber = fileNumStr.toInt();
-    if (fileNumber) {
-        m_saveLocation->u_numStartFrom->setValue(fileNumber + 1);
-    }
-
-    if (m_settingsUi.saveModeCB->currentIndex() == SaveModeManual) {
-        // Save last used dir, prefix and suffix.
-        m_saveLocation->u_urlRequester->setUrl(KIO::upUrl(fileUrl));
-        m_saveLocation->u_imgFormat->setCurrentText(QFileInfo(fileUrl.fileName()).suffix());
-    }
-}
-
-void Skanlite::getDir(void)
-{
-    QString dir = QFileDialog::getExistingDirectory(m_settingsDialog, QString(), m_settingsUi.saveDirLEdit->text());
-    if (!dir.isEmpty()) {
-        m_settingsUi.saveDirLEdit->setText(dir);
+    SkanliteImageSaver *imageSaver = qobject_cast<SkanliteImageSaver *>(sender());
+    if (imageSaver) {
+        imageSaver->deleteLater();
     }
 }
 
@@ -623,7 +626,7 @@ void readScannerOptions(const QString &groupName, QMap <QString, QString> &opts)
 void Skanlite::saveScannerOptions()
 {
     KConfigGroup saving(KSharedConfig::openConfig(), "Image Saving");
-    saving.writeEntry("NumberStartsFrom", m_saveLocation->u_numStartFrom->value());
+    saving.writeEntry("NumberStartsFrom", m_saveLocation->startNumber());
 
     if (!m_ksanew) {
         return;
@@ -641,13 +644,22 @@ void Skanlite::defaultScannerOptions()
         return;
     }
 
-    m_ksanew->setOptVals(m_defaultScanOpts);
+    applyScannerOptions(m_defaultScanOpts);
+}
+
+void Skanlite::applyScannerOptions(const QMap <QString, QString> &opts)
+{
+    if (m_ksanew->setOptVals(opts) == -1) {
+        m_pendingApplyScanOpts = opts;
+    } else {
+        m_pendingApplyScanOpts.clear();
+    }
 }
 
 void Skanlite::loadScannerOptions()
 {
     KConfigGroup saving(KSharedConfig::openConfig(), "Image Saving");
-    m_saveLocation->u_numStartFrom->setValue(saving.readEntry("NumberStartsFrom", 1));
+    m_saveLocation->setStartNumber(saving.readEntry("NumberStartsFrom", 1));
 
     if (!m_ksanew) {
         return;
@@ -655,13 +667,13 @@ void Skanlite::loadScannerOptions()
 
     QMap <QString, QString> opts;
     readScannerOptions(QString::fromLatin1("Options For %1").arg(m_deviceName), opts);
-    m_ksanew->setOptVals(opts);
+    applyScannerOptions(opts);
 }
 
 void Skanlite::availableDevices(const QList<KSaneWidget::DeviceInfo> &deviceList)
 {
     for (int i = 0; i < deviceList.size(); ++i) {
-        qDebug() << deviceList.at(i).name;
+        qCDebug(SKANLITE_LOG) << deviceList.at(i).name;
     }
 }
 
@@ -669,16 +681,16 @@ void Skanlite::alertUser(int type, const QString &strStatus)
 {
     switch (type) {
     case KSaneWidget::ErrorGeneral:
-        KMessageBox::sorry(0, strStatus, QLatin1String("Skanlite Test"));
+        KMessageBox::sorry(nullptr, strStatus, QLatin1String("Skanlite Test"));
         break;
     default:
-        KMessageBox::information(0, strStatus, QLatin1String("Skanlite Test"));
+        KMessageBox::information(nullptr, strStatus, QLatin1String("Skanlite Test"));
     }
 }
 
 void Skanlite::buttonPressed(const QString &optionName, const QString &optionLabel, bool pressed)
 {
-    qDebug() << "Button" << optionName << optionLabel << ((pressed) ? "pressed" : "released");
+    qCDebug(SKANLITE_LOG) << "Button" << optionName << optionLabel << ((pressed) ? "pressed" : "released");
 }
 
 // D-Bus interface related helper functions
@@ -688,7 +700,7 @@ QStringList serializeScannerOptions(const QMap<QString, QString> &opts)
     QStringList sl;
     QMap<QString, QString>::const_iterator it = opts.constBegin();
     while (it != opts.constEnd()) {
-        sl.append(it.key() + QLatin1String("=") + it.value());
+        sl.append(it.key() + QLatin1Char('=') + it.value());
         ++it;
     }
     return sl;
@@ -702,8 +714,8 @@ void deserializeScannerOptions(const QStringList &settings, QMap<QString, QStrin
     }
 }
 
-static const QStringList selectionSettings = { QLatin1String("tl-x"), QLatin1String("tl-y"),
-                                               QLatin1String("br-x"), QLatin1String("br-y") };
+static const auto selectionSettings = { QLatin1String("tl-x"), QLatin1String("tl-y"),
+                                        QLatin1String("br-x"), QLatin1String("br-y") };
 
 void filterSelectionSettings(QMap<QString, QString> &opts)
 {
@@ -748,7 +760,7 @@ void Skanlite::setScannerOptions(const QStringList &options, bool ignoreSelectio
     QMap <QString, QString> opts;
     deserializeScannerOptions(options, opts);
     processSelectionOptions(opts, ignoreSelection);
-    m_ksanew->setOptVals(opts);
+    applyScannerOptions(opts);
 }
 
 
@@ -764,20 +776,20 @@ void Skanlite::saveScannerOptionsToProfile(const QStringList &options, const QSt
     QMap <QString, QString> opts;
     deserializeScannerOptions(options, opts);
     processSelectionOptions(opts, ignoreSelection);
-    writeScannerOptions(QString(defaultProfileGroup).arg(m_deviceName).arg(profile), opts);
+    writeScannerOptions(QString(defaultProfileGroup).arg(m_deviceName, profile), opts);
 }
 
 void Skanlite::switchToProfile(const QString &profile, bool ignoreSelection)
 {
     QMap <QString, QString> opts;
-    readScannerOptions(QString(defaultProfileGroup).arg(m_deviceName).arg(profile), opts);
+    readScannerOptions(QString(defaultProfileGroup).arg(m_deviceName, profile), opts);
 
     if (opts.empty()) {
         opts = m_defaultScanOpts;
     }
 
     processSelectionOptions(opts, ignoreSelection);
-    m_ksanew->setOptVals(opts);
+    applyScannerOptions(opts);
 }
 
 void Skanlite::getDeviceName()
@@ -793,7 +805,7 @@ void Skanlite::getSelection()
     QStringList reply;
     foreach ( QString key, selectionSettings ) {
         if (opts.contains(key)) {
-            reply.append(key + QLatin1String("=") + opts[key]);
+            reply.append(key + QLatin1Char('=') + opts[key]);
         }
     }
     m_dbusInterface.setReply(reply);
