@@ -24,17 +24,19 @@
 #ifndef Skanlite_h
 #define Skanlite_h
 
-#include <QDir>
+#include <QUrl>
 #include <QDialog>
+#include <QTimer>
 
 #include <KSaneWidget>
 
 #include "ui_settings.h"
-#include "ImageViewer.h"
 #include "DBusInterface.h"
 
+class ShowImageDialog;
 class SaveLocation;
 class KAboutData;
+class QProgressBar;
 
 using namespace KSaneIface;
 
@@ -61,14 +63,16 @@ private:
 
 private Q_SLOTS:
     void showSettingsDialog();
-    void getDir();
     void imageReady(QByteArray &, int, int, int, int);
     void saveImage();
+    void updateSaveProgress();
+    void imageSaved(const QUrl &url, const QString &name, bool success);
     void showAboutDialog();
     void saveWindowSize();
 
     void saveScannerOptions();
     void defaultScannerOptions();
+    void applyScannerOptions(const QMap<QString, QString> &opts);
 
     void availableDevices(const QList<KSaneWidget::DeviceInfo> &deviceList);
 
@@ -93,14 +97,16 @@ protected:
 private:
     KAboutData              *m_aboutData;
     KSaneWidget             *m_ksanew = nullptr;
+    QProgressBar            *m_saveProgressBar = nullptr;
+    QUrl                     m_currentSaveUrl;
+    QTimer                   m_saveUpdateTimer;
     Ui::SkanliteSettings     m_settingsUi;
     QDialog                 *m_settingsDialog = nullptr;
-    QDialog                 *m_showImgDialog = nullptr;
-    // having this variable here is not so nice; ShowImgageDialog should be separate class
-    QPushButton             *m_showImgDialogSaveButton = nullptr;
+    ShowImageDialog         *m_showImgDialog = nullptr;
     SaveLocation            *m_saveLocation = nullptr;
     QString                  m_deviceName;
-    QMap<QString, QString>    m_defaultScanOpts;
+    QMap<QString, QString>   m_defaultScanOpts;
+    QMap<QString, QString>   m_pendingApplyScanOpts;
     QImage                   m_img;
     QByteArray               m_data;
     int                      m_width;
@@ -108,7 +114,6 @@ private:
     int                      m_bytesPerLine;
     int                      m_format;
 
-    ImageViewer              m_imageViewer;
     DBusInterface            m_dbusInterface;
     QStringList              m_filterList;
     QStringList              m_filter16BitList;
